'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _styledComponents = require('styled-components');

var _styledComponents2 = _interopRequireDefault(_styledComponents);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _Text = require('../Text');

var _Text2 = _interopRequireDefault(_Text);

var _theme = require('../../utils/theme');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var StyledAlert = _styledComponents2.default.div.withConfig({
  displayName: 'Alert__StyledAlert'
})(['display:flex;background:', ';border-radius:', ';border:2px solid ', ';color:', ';margin:8px 0 8px 0;min-height:1em;opacity:', ';padding:1em;transition:opacity 0.2s linear;a{color:', ';text-decoration:underline;&:hover{text-decoration:none;}}'], (0, _theme.fromAtoms)('Alert', 'type', 'background'), function (props) {
  return props.theme.borderRadius.soft;
}, (0, _theme.fromAtoms)('Alert', 'type', 'borderColor'), (0, _theme.fromAtoms)('Alert', 'type', 'color'), function (props) {
  return props.visible ? '1' : '0';
}, (0, _theme.fromAtoms)('Alert', 'type', 'color'));

var Content = _styledComponents2.default.div.withConfig({
  displayName: 'Alert__Content'
})(['flex-grow:1;']);

var Title = _styledComponents2.default.div.withConfig({
  displayName: 'Alert__Title'
})(['margin-bottom:0.5em;']);

var Icon = _styledComponents2.default.i.withConfig({
  displayName: 'Alert__Icon'
})(['margin-right:0.5em;']);

var CloseIcon = _styledComponents2.default.i.withConfig({
  displayName: 'Alert__CloseIcon'
})(['cursor:pointer;display:', ';margin-left:1em;'], function (props) {
  return props.visible ? 'block' : 'none';
});

var normaliseIconName = function normaliseIconName(name) {
  return name.indexOf('fa-') === -1 ? 'fa-' + name : name;
};

/**
 * An alert to let the user know about an action or state.
 * ```javascript
 * import { Alert } from 'weaveworks-ui-components';
 *
 * export default function() {
 *  return (
 *    <div>
 *     <Alert type="warning" visible={true}>
 *       Warning: Hmm, this is not good, but it is not terrible.
 *     </Alert>
 *    </div>
 *   )
 * }
 * ```
 */
function Alert(props) {
  var children = props.children,
      icon = props.icon,
      onClose = props.onClose,
      title = props.title,
      visible = props.visible;


  return _react2.default.createElement(
    StyledAlert,
    props,
    _react2.default.createElement(
      Content,
      null,
      title && _react2.default.createElement(
        Title,
        null,
        icon && _react2.default.createElement(Icon, { className: 'fa ' + normaliseIconName(icon) }),
        _react2.default.createElement(
          _Text2.default,
          { normal: true, bold: true },
          title
        )
      ),
      children
    ),
    onClose && _react2.default.createElement(CloseIcon, {
      visible: visible,
      onClick: onClose,
      className: 'fa fa-remove'
    })
  );
}

Alert.propTypes = {
  /**
   * Show an icon for the title
   */
  icon: _propTypes2.default.string,
  /**
   * Show a title for the alert
   */
  title: _propTypes2.default.string,
  /**
   * Set the color scheme to indicate the nature of the alert.
   */
  type: _propTypes2.default.oneOf(['default', 'info', 'success', 'warning', 'error']),
  /**
   * Toggle whether the alert is shown
   */
  visible: _propTypes2.default.bool,
  /**
   * Callback that runs with the alert is dismissed by clicking the 'X'
   */
  onClose: _propTypes2.default.func
};

Alert.defaultProps = {
  icon: '',
  title: '',
  type: 'default',
  visible: true
};

exports.default = Alert;