'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _each2 = require('lodash/each');

var _each3 = _interopRequireDefault(_each2);

var _noop2 = require('lodash/noop');

var _noop3 = _interopRequireDefault(_noop2);

var _times2 = require('lodash/times');

var _times3 = _interopRequireDefault(_times2);

var _isEmpty2 = require('lodash/isEmpty');

var _isEmpty3 = _interopRequireDefault(_isEmpty2);

var _filter2 = require('lodash/filter');

var _filter3 = _interopRequireDefault(_filter2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _styledComponents = require('styled-components');

var _styledComponents2 = _interopRequireDefault(_styledComponents);

var _d3Shape = require('d3-shape');

var _reactMotion = require('react-motion');

var _dom = require('../../utils/dom');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function weakSpring(value) {
  return (0, _reactMotion.spring)(value, { stiffness: 100, damping: 18, precision: 1 });
}

var spline = (0, _d3Shape.line)().curve(_d3Shape.curveBasis).x(function (d) {
  return d.x;
}).y(function (d) {
  return d.y;
});

var EdgeShadow = _styledComponents2.default.path.withConfig({
  displayName: 'GraphEdge__EdgeShadow'
})(['stroke:', ';stroke-width:', ';stroke-opacity:0;fill:none;', ';'], function (props) {
  return props.theme.colors.blue400;
}, function (props) {
  return 10 * props.thickness;
}, function (props) {
  return props.highlighted && '\n      stroke-opacity: ' + (props.contrastMode ? 0.3 : 0.1) + ';\n    ';
});

var EdgeDotted = _styledComponents2.default.path.withConfig({
  displayName: 'GraphEdge__EdgeDotted'
})(['stroke:', ';stroke-dasharray:1,30;stroke-linecap:round;stroke-width:5;fill:none;'], function (props) {
  return props.contrastMode ? props.theme.colors.black : props.theme.colors.purple500;
});

var EdgeLine = _styledComponents2.default.path.withConfig({
  displayName: 'GraphEdge__EdgeLine'
})(['stroke:', ';stroke-width:', ';fill:none;'], function (props) {
  return props.contrastMode ? props.theme.colors.black : props.theme.colors.purple500;
}, function (props) {
  return props.thickness;
});

var EdgeArrowPolygon = _styledComponents2.default.polygon.withConfig({
  displayName: 'GraphEdge__EdgeArrowPolygon'
})(['fill:', ';'], function (props) {
  return props.contrastMode ? props.theme.colors.black : props.theme.colors.purple500;
});

var EdgeArrowDefinition = function EdgeArrowDefinition(_ref) {
  var id = _ref.id,
      offset = _ref.offset,
      thickness = _ref.thickness,
      contrastMode = _ref.contrastMode;
  return _react2.default.createElement(
    'defs',
    null,
    _react2.default.createElement(
      'marker',
      {
        id: id,
        refY: '3.5',
        refX: offset / thickness,
        markerWidth: 15 / thickness,
        markerHeight: 15 / thickness,
        viewBox: '1 0 10 10',
        orient: 'auto'
      },
      _react2.default.createElement(EdgeArrowPolygon, { points: '0 0, 10 3.5, 0 7', contrastMode: contrastMode })
    )
  );
};

// Converts a waypoints map of the format { x0: 11, y0: 22, x1: 33, y1: 44 }
// that is used by Motion to an array of waypoints in the format
// [{ x: 11, y: 22 }, { x: 33, y: 44 }] that can be used by D3.
var waypointsMapToArray = function waypointsMapToArray(waypointsMap) {
  var waypointsArray = (0, _times3.default)(10, function () {
    return {};
  });
  (0, _each3.default)(waypointsMap, function (value, key) {
    var _ref2 = [key[0], key.slice(1)],
        axis = _ref2[0],
        index = _ref2[1];

    waypointsArray[index][axis] = value;
  });
  return (0, _filter3.default)(waypointsArray, function (e) {
    return !(0, _isEmpty3.default)(e);
  });
};

// Converts a waypoints array of the input format [{ x: 11, y: 22 }, { x: 33, y: 44 }]
// to an array of waypoints that is used by Motion in the format { x0: 11, y0: 22, x1: 33, y1: 44 }.
var waypointsArrayToMap = function waypointsArrayToMap(waypointsArray) {
  var waypointsMap = {};
  (0, _each3.default)(waypointsArray, function (waypoint, index) {
    waypointsMap['x' + index] = weakSpring(waypoint.x);
    waypointsMap['y' + index] = weakSpring(waypoint.y);
  });
  return waypointsMap;
};

/**
 * A component for rendering edges that connect the graph nodes.
 */

var GraphEdge = function (_React$Component) {
  _inherits(GraphEdge, _React$Component);

  function GraphEdge() {
    var _ref3;

    var _temp, _this, _ret;

    _classCallCheck(this, GraphEdge);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref3 = GraphEdge.__proto__ || Object.getPrototypeOf(GraphEdge)).call.apply(_ref3, [this].concat(args))), _this), _this.state = {
      waypointsMap: []
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(GraphEdge, [{
    key: 'componentWillMount',
    value: function componentWillMount() {
      if (this.props.isAnimated) {
        this.prepareWaypointsForMotion(this.props);
      }
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      // immutablejs allows us to `===`! \o/
      var waypointsChanged = this.props.waypoints !== nextProps.waypoints;
      var animationChanged = this.props.isAnimated !== nextProps.isAnimated;
      if (waypointsChanged || animationChanged) {
        this.prepareWaypointsForMotion(nextProps);
      }
    }
  }, {
    key: 'prepareWaypointsForMotion',
    value: function prepareWaypointsForMotion(_ref4) {
      var waypoints = _ref4.waypoints,
          waypointsCap = _ref4.waypointsCap,
          isAnimated = _ref4.isAnimated;

      // Don't update if the edges are not animated.
      if (!isAnimated) return;

      // The Motion library requires the number of waypoints to be constant, so we fill in for
      // the missing ones by reusing the edge source point, which doesn't affect the edge shape
      // because of how the curveBasis interpolation is done.
      var waypointsMissing = waypointsCap - waypoints.size;
      if (waypointsMissing > 0) {
        waypoints = (0, _times3.default)(waypointsMissing, function () {
          return waypoints[0];
        }).concat(waypoints);
      }

      this.setState({ waypointsMap: waypointsArrayToMap(waypoints) });
    }
  }, {
    key: 'renderEdge',
    value: function renderEdge(props) {
      var id = props.id,
          waypoints = props.waypoints,
          thickness = props.thickness,
          withArrow = props.withArrow,
          arrowOffset = props.arrowOffset,
          isDotted = props.isDotted,
          highlighted = props.highlighted,
          contrastMode = props.contrastMode;


      var encodedId = (0, _dom.encodeIdAttribute)(id);
      var encodedArrowId = 'end-arrow-' + encodedId;
      var arrowThickness = Math.sqrt(thickness);
      var path = spline(waypoints);

      return _react2.default.createElement(
        'g',
        {
          id: encodedId,
          onMouseEnter: function onMouseEnter(ev) {
            return props.onMouseEnter(id, ev);
          },
          onMouseLeave: function onMouseLeave(ev) {
            return props.onMouseLeave(id, ev);
          }
        },
        withArrow && _react2.default.createElement(EdgeArrowDefinition, {
          id: encodedArrowId,
          thickness: arrowThickness,
          offset: arrowOffset,
          contrastMode: contrastMode
        }),
        _react2.default.createElement(EdgeShadow, {
          d: path,
          highlighted: highlighted,
          thickness: thickness,
          contrastMode: contrastMode
        }),
        isDotted && _react2.default.createElement(EdgeDotted, { d: path, contrastMode: contrastMode }),
        _react2.default.createElement(EdgeLine, {
          d: path,
          thickness: thickness,
          contrastMode: contrastMode,
          markerEnd: withArrow ? 'url(#' + encodedArrowId + ')' : null
        })
      );
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      if (!this.props.isAnimated) {
        return this.renderEdge(this.props);
      }

      return (
        // For the Motion interpolation to work, the waypoints need to be in a map format like
        // { x0: 11, y0: 22, x1: 33, y1: 44 } that we convert to the array format when rendering.
        _react2.default.createElement(
          _reactMotion.Motion,
          {
            style: _extends({
              interpolatedThickness: weakSpring(this.props.thickness)
            }, this.state.waypointsMap)
          },
          function (_ref5) {
            var interpolatedThickness = _ref5.interpolatedThickness,
                interpolatedWaypoints = _objectWithoutProperties(_ref5, ['interpolatedThickness']);

            return _this2.renderEdge(_extends({}, _this2.props, {
              waypoints: waypointsMapToArray(interpolatedWaypoints),
              thickness: interpolatedThickness
            }));
          }
        )
      );
    }
  }]);

  return GraphEdge;
}(_react2.default.Component);

GraphEdge.propTypes = {
  /**
   * A unique edge ID
   */
  id: _propTypes2.default.string.isRequired,
  /**
   * A list of points in the { x, y } format describing the edge path
   */
  waypoints: _propTypes2.default.arrayOf(_propTypes2.default.shape({
    x: _propTypes2.default.number.isRequired,
    y: _propTypes2.default.number.isRequired
  })).isRequired,
  /**
   * A number of waypoints to cap to in case the edge is animated
   */
  waypointsCap: _propTypes2.default.number,
  /**
   * Thickness of the rendered edge line
   */
  thickness: _propTypes2.default.number,
  /**
   * Shows the blue shadow around the edge if true
   */
  highlighted: _propTypes2.default.bool,
  /**
   * Draws a one-way arrow on the edge if true
   */
  withArrow: _propTypes2.default.bool,
  /**
   * Distance from the target point (tweak this to pull the arrow out of the rendered node)
   */
  arrowOffset: _propTypes2.default.number,
  /**
   * Shows extra dots on the edge path if true
   */
  isDotted: _propTypes2.default.bool,
  /**
   * Animates the edge motion if true
   */
  isAnimated: _propTypes2.default.bool,
  /**
   * Renders the edge in a high contrast mode
   */
  contrastMode: _propTypes2.default.bool,
  /**
   * Callback for mouse pointer entering the edge
   */
  onMouseEnter: _propTypes2.default.func,
  /**
   * Callback for mouse pointer leaving the edge
   */
  onMouseLeave: _propTypes2.default.func
};

GraphEdge.defaultProps = {
  waypointsCap: 10,
  thickness: 1,
  highlighted: false,
  withArrow: false,
  arrowOffset: 0,
  isDotted: false,
  isAnimated: false,
  contrastMode: false,
  onMouseEnter: _noop3.default,
  onMouseLeave: _noop3.default
};

exports.default = GraphEdge;